
#ifndef _SCP_H
#define _SCP_H
#include "osihfe.h"
#include "hfe.h"
#include <stdio.h>
///SCP support

//NOTE regarding 40 track format:
//Commodore GCR (1541/1571/compatibles), Apple II, and other disks that are 40 track
//formats can have half-tracks between the full tracks.  To support this, ALL single
//sided 40 track disk types have the start and end track numbers doubled, and 80 tracks of
//space are used.  So, each track number represents either a full track or a half-track.


typedef struct scptrackdataheader_	   //TDH
{
	unsigned char HEADERSIGNATURE[3]; // 0x00 to 0x02, "TRK"
	unsigned char tracknum;		 //0x03 For single sided disks, tracks 0-41 (48TPI) or 0-83 (96TPI) are used.  
								 //     For double-sided disks, the actual track number is this value divided by 2.  
								 //     The remainder (0 or 1) is the head.  0 represents the bottom head and 1 represents the top head. 

	unsigned long  indextime;	 //0x04-0x07 LE duration of 1 revolution in samples ex: (6666667) * 25ns = 166666675ns = 166.666675ms,  
	                             //          60000ms per min / 166.666675 = 360 RPM
	unsigned long  tracklen;		 //0x08-0x0B LE length of track in bitcells 16bits = 2 & bytelen
	unsigned long  trackoffset;	 //0x0C-0x0F LE offset from start of TDH to flux data
} scptrackdataheader;

//If there were more revolutions, there would be three more longwords stored for each
//additional revolution.. example:
//BYTES 0x10-0x1B would be the 2nd entry (duration2, length2, offset2).
//BYTES 0x1C-0x27 would be the 3rd entry (duration3, length3, offset3).
//BYTES 0x28-0x33 would be the 4th entry (duration4, length4, offset4).
//BYTES 0x34-0x3F would be the 5th entry (duration5, length5, offset5).

//Flux data for each revolution is always stored sequentially (like the disk is spinning multiple times)
// TRACK DATA = 16 BIT VALUE, TIME IN NANOSECONDS/25ns FOR ONE BIT CELL TIME
// i.e. 0x00DA = 218, 218*25 = 5450ns = 5.450us
// if bitcell time is 0 the time overflowed (65536*25ns) then the next bitcell time will be added to 65536
// until a non 0 entry is encountered
// i.e. 0x0000, 0x0000, 0x7FFF = 65536 + 65536 + 32767 = 163839*25 = 4095975ns
// The number of bitcells only increases by the number of entries, and is not affected by the overall time.  So, in above 
// example even though the time could be what thousands of bitcells times would normally be, the number of bitcells might 
// only be 3 entries

typedef struct scpfileformatheader_
{
unsigned char HEADERSIGNATURE[3]; // "SCP"
unsigned char version;			// Revision 0.0
unsigned char disktype;			// Disk Class & Manufacturer Class. 0x80 = other/360K or 0x84 other/720K?
unsigned char revs;				// revolutions per track (number of track samples) (3+)
unsigned char start_track;		// starting track
unsigned char end_track;		// ending track (2x actual track if using half tracks) max 168
unsigned char scpflag;			// flags - see below
unsigned char bitcellwidth;		// 0=16bits, else value = # of bits for sample
unsigned char heads;			// 0=2 sided, 1= side 0 (bottom), 2= side 1 (top)
unsigned char resolution;		// 0=25ns else (25 + 25*resolution) ns
unsigned int  checksum;	//little endian 32bit checksum offset 0x10 to end of file except if scpflag bit 4 set
long TDH[1];  //table of offsets to track data header * number of tracks
              //If no flux data for a track is present, then the entry will contain a longword of zeros (0x00000000)
} scpfileformatheader;

//scpflag:
//bit 0 INDEX, when set indicates track data starts at index detection
//bit 1 TPI,  clear = 48TPI, set = 96TPI
//bit 2 RPM,  clear = 300RPM, set=360 RPM
//bit 3 TYPE, clear for preservation, set for normalization
//bit 4 MODE, clear = read-only, set = R/W
//bit 5 FOOTER, clear if not extension footer, set if extension footer present
//bit 6 EXTENDED MODE, clear for floppy drives, set for other media

int CreateHFEFromSCP( FILE* inFile, unsigned char **ppHFE, int* pnLen, int nSkip = 1, int nSCPRev = 1);
bool DisplaySCPFileInfo( FILE* inFile);

#endif //_SCP_H
